;; Step/theme.jl

;; Copyright (C) 1999 Michele Campeotto <micampe@interpuntonet.it>

;; This theme is based on Window Maker, another window manager for X11
;; (wich is based on Apple NeXTStep), I ported it to Sawmill because I liked
;; his look'n'feel but Window Maker is'nt working well with Gnome.

;; --- Frame preferences ---------------------------------------------------

(defgroup Step-frame "Step theme"
  :group appearance)

(defcustom Step:title-alignement 'center
  "Alignement of the text in the titlebar"
  :group (appearance Step-frame)
  :type symbol
  :options (left center right)
  :after-set after-setting-frame-option)

(defcustom Step:flat-buttons nil
  "Use flat buttons on titlebar"
  :group (appearance Step-frame)
  :type boolean
  :after-set after-setting-frame-option)

(defcustom Step:titlebar-active-color (get-color "#3d70785191ea")
  "Color of the active window titlebar"
  :group (appearance Step-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step:titlebar-inactive-color (get-color "#bfffbffbbffb")
  "Color of inactive windows titlebars"
  :group (appearance Step-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step:title-active-color (get-color "white")
  "Color of the active window title"
  :group (appearance Step-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step:title-inactive-color (get-color "gray25")
  "Color of inactive windows titles"
  :group (appearance Step-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step:resizebar-active-color (get-color "gray75")
  "Color of the active window resizebar"
  :group (appearance Step-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step:resizebar-inactive-color (get-color "gray75")
  "Color of inactive windows resizebars"
  :group (appearance Step-frame)
  :type color
  :after-set after-setting-frame-option)

;; --- Frame definition ----------------------------------------------------

(let* (
	(title-font (get-font "-*-helvetica-bold-r-normal-*-12-*-*-*-*-*-*"))
	(title-colors (lambda () (list Step:title-inactive-color Step:title-active-color)))
	(titlebar-colors (lambda () (list Step:titlebar-inactive-color Step:titlebar-active-color)))
	(resizebar-colors (lambda () (list Step:resizebar-inactive-color Step:resizebar-active-color)))

	(recolor-image
		(lambda (img color)
			(when (and (imagep img) (colorp color))
				(let*
					((rgb (color-rgb color))
					 (red (nth 0 rgb))
					 (green (nth 1 rgb))
					 (blue (nth 2 rgb)))
				(set-image-modifier img nil '(255 255 255))
				(set-image-modifier img 'red (list (/ red 256) 255 (/ red 256)))
				(set-image-modifier img 'green (list (/ green 256) 255 (/ green 256)))
				(set-image-modifier img 'blue (list (/ blue 256) 255 (/ blue 256))))
			)))

	(brighten-color 
		(lambda (color delta)
			(get-color-rgb
                                (min 65535 (+ (car (color-rgb color)) delta))
                                (min 65535 (+ (cadr (color-rgb color)) delta))
                                (min 65535 (+ (car (cddr (color-rgb color))) delta))
			)))

	(darken-color 
		(lambda (color delta)
			(get-color-rgb
                                (max 0 (- (car (color-rgb color)) delta))
                                (max 0 (- (cadr (color-rgb color)) delta))
                                (max 0 (- (car (cddr (color-rgb color))) delta))
			)))

	(minimize-flat (list
		(recolor-image (make-image "minimize-flat-button.png")
			Step:title-inactive-color)
		(recolor-image (make-image "minimize-flat-button.png")
			Step:title-active-color) nil
		(make-image "minimize-flat-pressed.png")))

	(minimize-bevel (list
		(make-image "minimize-bevel-button.png")
		(make-image "minimize-bevel-button.png") nil
		(make-image "minimize-bevel-pressed.png")))

	(minimize (lambda ()
		(if Step:flat-buttons minimize-flat minimize-bevel)))

	(close-flat (list
		(recolor-image (make-image "close-flat-button.png")
			Step:title-inactive-color)
		(recolor-image (make-image "close-flat-button.png")
			Step:title-active-color) nil
		(make-image "close-flat-pressed.png")))

	(close-bevel (list
		(make-image "close-bevel-button.png")
		(make-image "close-bevel-button.png") nil
		(make-image "close-bevel-pressed.png")))

	(close (lambda ()
		(if Step:flat-buttons close-flat close-bevel)))

	(border-color (get-color "black"))

	(bevel-up-left-color (list 
		(brighten-color Step:titlebar-inactive-color 28000)
		(brighten-color Step:titlebar-active-color 30000)))
	(bevel-down-right-color (list
		(darken-color Step:titlebar-inactive-color 28000)
		(darken-color Step:titlebar-active-color 30000)))

	(bevel-resize-up-left-color (list
		(brighten-color Step:resizebar-inactive-color 22000)
		(brighten-color Step:resizebar-active-color 22000)))
	(bevel-resize-down-right-color (list
		(darken-color Step:resizebar-inactive-color 22000)
		(darken-color Step:resizebar-active-color 22000)))

;; --- Frame parts ------------------------
;; Titlebar black border
	(black-left `(
		(background . ,border-color)
		(left-edge . -1)
		(width . 1)
		(top-edge . -23)
		(height . 23)))
	(black-right `(
		(background . ,border-color)
		(right-edge . -1)
		(width . 1)
		(top-edge . -23)
		(height . 23)))
	(black-top `(
		(background . ,border-color)
		(right-edge . -1)
		(left-edge . -1)
		(top-edge . -23)
		(height . 1)))
	(black-bottom `(
		(background . ,border-color)
		(right-edge . -1)
		(left-edge . -1)
		(top-edge . -1)
		(height . 1)))
;; Title bevel border
	(bevel-left `(
		(background . ,bevel-up-left-color)
		(left-edge . 0)
		(width . 1)
		(top-edge . -22)
		(height . 21)))
	(bevel-top `(
		(background . ,bevel-up-left-color)
		(right-edge . 0)
		(left-edge . 0)
		(top-edge . -22)
		(height . 1)))
	(bevel-right `(
		(background . ,bevel-down-right-color)
		(right-edge . 0)
		(width . 1)
		(top-edge . -22)
		(height . 21)))
	(bevel-bottom `(
		(background . ,bevel-down-right-color)
		(right-edge . 0)
		(left-edge . 1)
		(top-edge . -2)
		(height . 1)))

;; Titlebar
	(titlebar `(
		(background . ,titlebar-colors)
		(left-edge . ,(if Step:flat-buttons 23 20))
		(right-edge . ,(if Step:flat-buttons 23 20))
		(top-edge . -21)
		(height . 19)
		(cursor . left_ptr)
		(class . title)))
	(titlebar-text `(
		(background . ,titlebar-colors)
		(foreground . ,title-colors)
		(font . ,title-font)
		(text . ,window-name)
		(x-justify . ,Step:title-alignement)
		(y-justify . center)
		(left-edge . ,(if Step:flat-buttons 28 23))
		(right-edge . ,(if Step:flat-buttons 28 23))
		(top-edge . -21)
		(height . 19)
		(cursor . left_ptr)
		(class . title)))

;; Transient window titlebar
	(transient-titlebar `(
		(background . ,titlebar-colors)
		(left-edge . 1)
		(right-edge . ,(if Step:flat-buttons 23 20))
		(top-edge . -21)
		(height . 19)
		(cursor . left_ptr)
		(class . title)))
	(transient-titlebar-text `(
		(background . ,titlebar-colors)
		(foreground . ,title-colors)
		(font . ,title-font)
		(text . ,window-name)
		(x-justify . ,Step:title-alignement)
		(y-justify . center)
		(left-edge . ,6)
		(right-edge . ,(if Step:flat-buttons 28 23))
		(top-edge . -21)
		(height . 19)
		(cursor . left_ptr)
		(class . title)))

;; Minimize button right border
	(button-border-11 
		(if Step:flat-buttons `(
			(background . ,bevel-down-right-color)
			(left-edge . 20)
			(width . 1)
			(top-edge . -21)
			(height . 20))
			nil))
	(button-border-12
		(if Step:flat-buttons `(
			(background . ,border-color)
			(left-edge . 21)
			(width . 1)
			(top-edge . -22)
			(height . 21))
			nil))
	(button-border-13
		(if Step:flat-buttons `(
			(background . ,bevel-up-left-color)
			(left-edge . 22)
			(width . 1)
			(top-edge . -22)
			(height . 21))
			nil))

;; Minimize button icon
	(minimize `(
		(background . ,titlebar-colors)
		(foreground . ,minimize)
		(left-edge . 1)
		(width . 19)
		(top-edge . -21)
		(height . 19)
		(cursor . left_ptr)
		(class . iconify-button)))

;; Close button left border
	(button-border-21 
		(if Step:flat-buttons `(
			(background . ,bevel-down-right-color)
			(right-edge . 22)
			(width . 1)
			(top-edge . -21)
			(height . 20))
			nil))
	(button-border-22
		(if Step:flat-buttons `(
			(background . ,border-color)
			(right-edge . 21)
			(width . 1)
			(top-edge . -22)
			(height . 21))
			nil))
	(button-border-23
		(if Step:flat-buttons `(
			(background . ,bevel-up-left-color)
			(right-edge . 20)
			(width . 1)
			(top-edge . -22)
			(height . 21))
			nil))

;; Close button icon
	(close `(
		(background . ,titlebar-colors)
		(foreground . ,close)
		(right-edge . 1)
		(width . 19)
		(top-edge . -21)
		(height . 19)
		(cursor . left_ptr)
		(class . close-button)))

;; Left border
	(left-border `(
		(background . ,border-color)
		(right-edge . -1)
		(width . 1)
		(top-edge . 0)
		(bottom-edge . -1)
;;		(cursor . left_ptr)
		(class . right-border)))

;;Right border
	(right-border `(
		(background . ,border-color)
		(left-edge . -1)
		(width . 1)
		(top-edge . 0)
		(bottom-edge . -1)
;;		(cursor . left_ptr)
		(class . left-border)))

;; --- Frame definition -------------------

;; Normal window frame
	(frame `(
		,black-left
		,black-right
		,black-top
		,black-bottom
		,bevel-left
		,bevel-right
		,bevel-top
		,bevel-bottom
		,titlebar
		,titlebar-text
		,button-border-11
		,button-border-12
		,button-border-13
		,minimize
		,button-border-21
		,button-border-22
		,button-border-23
		,close
		,left-border
		,right-border
	;; Bottom left corner
		((background . ,border-color)
		 (left-edge . -1)
		 (width . 1)
		 (bottom-edge . -9)
		 (height . 9)
;;		 (cursor . left_ptr)
		 (class . left-border))
		((background . ,bevel-resize-down-right-color)
		 (left-edge . 29)
		 (width . 1)
		 (bottom-edge . -8)
		 (height . 6))
		((background . ,bevel-resize-up-left-color)
		 (left-edge . 30)
		 (width . 1)
		 (bottom-edge . -8)
		 (height . 6))
		((background . ,resizebar-colors)
		 (left-edge . 0)
		 (width . 29)
		 (bottom-edge . -8)
		 (height . 6)
;;		 (cursor . left_ptr)
		 (class . bottom-left-corner))
	;; Bottom border
		((background . ,bevel-resize-down-right-color)
		 (right-edge . 0)
		 (left-edge . 0)
		 (bottom-edge . -1)
		 (height . 1))
		((background . ,bevel-resize-up-left-color)
		 (right-edge . 0)
		 (left-edge . 0)
		 (bottom-edge . -2)
		 (height . 1))
		((background . ,border-color)
		 (right-edge . -1)
		 (left-edge . -1)
		 (bottom-edge . -9)
		 (height . 1))
		((background . ,resizebar-colors)
		 (left-edge . 31)
		 (right-edge . 31)
		 (bottom-edge . -8)
		 (height . 6)
;;		 (cursor . left_ptr)
		 (class . bottom-border))
	;; Bottom right corner
		((background . ,border-color)
		 (right-edge . -1)
		 (width . 1)
		 (bottom-edge . -9)
		 (height . 9)
;;		 (cursor . left_ptr)
		 (class . left-border))
		((background . ,bevel-resize-down-right-color)
		 (right-edge . 30)
		 (width . 1)
		 (bottom-edge . -8)
		 (height . 6))
		((background . ,bevel-resize-up-left-color)
         (right-edge . 29)
		 (width . 1)
		 (bottom-edge . -8)
		 (height . 6))
		((background . ,resizebar-colors)
		 (right-edge . 0)
		 (width . 29)
		 (bottom-edge . -8)
		 (height . 6)
;;		 (cursor . left_ptr)
		 (class . bottom-right-corner))
	)) ;; end frame

;; Normal window shaded frame
	(shaped-frame `(
		,black-left
		,black-right
		,black-top
		,black-bottom
		,bevel-left
		,bevel-right
		,bevel-top
		,bevel-bottom
		,titlebar
		,titlebar-text
		,button-border-11
		,button-border-12
		,button-border-13
		,minimize
		,button-border-21
		,button-border-22
		,button-border-23
		,close
	)) ;; end shaped-frame

;; Dialog window frame
	(transient-frame `(
		,black-left
		,black-right
		,black-top
		,black-bottom
		,bevel-left
		,bevel-right
		,bevel-top
		,bevel-bottom
		,transient-titlebar
		,transient-titlebar-text
		,button-border-21
		,button-border-22
		,button-border-23
		,close
		,left-border
		,right-border
	;; Bottom border
		((background . ,border-color)
		 (left-edge . 0)
		 (right-edge . 0)
		 (bottom-edge . -1)
		 (height . 1)
;;		 (cursor . left_ptr)
		 (class . bottom-border))
	)) ;; end transient-frame

;; Dialog window shaded frame
	(shaped-transient-frame `(
		,black-left
		,black-right
		,black-top
		,black-bottom
		,bevel-left
		,bevel-right
		,bevel-top
		,bevel-bottom
		,transient-titlebar
		,transient-titlebar-text
		,button-border-21
		,button-border-22
		,button-border-23
		,close
	))) ;; end shaped-transient-frame

    (add-frame-style 'Step
        (lambda (w type)
            (case type
                ((default) frame)
                ((transient) transient-frame)
                ((shaped) shaped-frame)
                ((shaped-transient) shaped-transient-frame)
    )))
)
